"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const aws_amplify_1 = require("aws-amplify");
const react_hooks_1 = require("@testing-library/react-hooks");
const constants_1 = require("../constants");
const useDataStoreUpdateAction_1 = require("../useDataStoreUpdateAction");
const amplify_symbol_1 = require("../../amplify-symbol");
const models_1 = require("./test-models/models");
jest.mock('aws-amplify');
const name = 'milk';
const id = '1234';
const updateActionArgs = {
    model: models_1.Todo,
    id,
    fields: { name },
};
const saveSpy = jest.spyOn(aws_amplify_1.DataStore, 'save');
const querySpy = jest.spyOn(aws_amplify_1.DataStore, 'query').mockImplementation(() => Promise.resolve([{ id, name }]));
const hubDispatchSpy = jest.spyOn(aws_amplify_1.Hub, 'dispatch');
describe('useAuthSignOutAction', () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });
    it('should call DataStore.save', async () => {
        const { result: { current: action }, } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreUpdateAction_1.useDataStoreUpdateAction)(updateActionArgs));
        await action();
        expect(querySpy).toHaveBeenCalledTimes(1);
        expect(saveSpy).toHaveBeenCalledTimes(1);
    });
    it('should call Hub with started and finished events', async () => {
        const { result: { current: action }, } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreUpdateAction_1.useDataStoreUpdateAction)(updateActionArgs));
        await action();
        expect(hubDispatchSpy).toHaveBeenCalledTimes(2);
        expect(hubDispatchSpy).toHaveBeenCalledWith(constants_1.UI_CHANNEL, {
            data: { id, fields: { name } },
            event: constants_1.ACTION_DATASTORE_UPDATE_STARTED,
        }, constants_1.EVENT_ACTION_DATASTORE_UPDATE, amplify_symbol_1.AMPLIFY_SYMBOL);
        expect(hubDispatchSpy).toHaveBeenCalledWith(constants_1.UI_CHANNEL, {
            data: { id, fields: { name } },
            event: constants_1.ACTION_DATASTORE_UPDATE_FINISHED,
        }, constants_1.EVENT_ACTION_DATASTORE_UPDATE, amplify_symbol_1.AMPLIFY_SYMBOL);
    });
    it('should call Hub with error message if DataStore.save rejects', async () => {
        const errorMessage = 'Invalid data model';
        saveSpy.mockImplementation(() => Promise.reject(new Error(errorMessage)));
        querySpy.mockImplementation(() => Promise.resolve([{ id, name }]));
        const { result: { current: action }, } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreUpdateAction_1.useDataStoreUpdateAction)(updateActionArgs));
        await action();
        expect(hubDispatchSpy).toHaveBeenCalledTimes(2);
        expect(hubDispatchSpy).toHaveBeenCalledWith(constants_1.UI_CHANNEL, {
            data: { id, fields: { name } },
            event: constants_1.ACTION_DATASTORE_UPDATE_STARTED,
        }, constants_1.EVENT_ACTION_DATASTORE_UPDATE, amplify_symbol_1.AMPLIFY_SYMBOL);
        expect(hubDispatchSpy).toHaveBeenCalledWith(constants_1.UI_CHANNEL, {
            data: {
                id,
                fields: { name },
                errorMessage,
            },
            event: constants_1.ACTION_DATASTORE_UPDATE_FINISHED,
        }, constants_1.EVENT_ACTION_DATASTORE_UPDATE, amplify_symbol_1.AMPLIFY_SYMBOL);
    });
    it('when original not found, should call Hub with error message', async () => {
        const { result: { current: action }, } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreUpdateAction_1.useDataStoreUpdateAction)(updateActionArgs));
        querySpy.mockImplementationOnce(() => Promise.resolve(undefined));
        await action();
        expect(hubDispatchSpy).toHaveBeenCalledTimes(2);
        expect(hubDispatchSpy).toHaveBeenCalledWith(constants_1.UI_CHANNEL, {
            data: { id, fields: { name } },
            event: constants_1.ACTION_DATASTORE_UPDATE_STARTED,
        }, constants_1.EVENT_ACTION_DATASTORE_UPDATE, amplify_symbol_1.AMPLIFY_SYMBOL);
        expect(hubDispatchSpy).toHaveBeenCalledWith(constants_1.UI_CHANNEL, {
            data: {
                id,
                fields: { name },
                errorMessage: `Error querying datastore item by id: ${id}`,
            },
            event: constants_1.ACTION_DATASTORE_UPDATE_FINISHED,
        }, constants_1.EVENT_ACTION_DATASTORE_UPDATE, amplify_symbol_1.AMPLIFY_SYMBOL);
    });
});
//# sourceMappingURL=useDataStoreUpdateAction.test.js.map